async (options) => {
    const { deep, data: { newLink: parseItLink }, } = options;
    const { default: ts } = await import("typescript");
    const { default: util } = await import("util");
    try {
        const result = await main();
        return {
            result: util.inspect(result),
        };
    }
    catch (error) {
        throw {
            error: util.inspect(error),
        };
    }
    async function main() {
        const { data: [rootLink], } = await deep.select({
            id: parseItLink.from_id,
        });
        if (!rootLink) {
            throw new Error(`parseIt.to does not exist: ##${parseItLink.from_id}`);
        }
        const obj = rootLink.value?.value;
        if (!obj) {
            throw new Error(`##${rootLink.id} must have value`);
        }
        const clientHandlerResult = await callClientHandler({
            deep,
            linkId: await deep.id(deep.linkId, "clientHandler"),
            args: [
                {
                    deep: deep,
                    obj: obj,
                    rootLinkId: rootLink.id,
                    resultLinkId: parseItLink.to_id,
                    // TODO?
                    // customMethods: options.customMethods,
                },
            ],
        });
        return clientHandlerResult;
    }
    async function callClientHandler(options) {
        const { linkId, deep, args } = options;
        const code = await deep
            .select({
            in: {
                id: linkId,
            },
        })
            .then((result) => {
            const link = result.data[0];
            if (!link)
                throw new Error(`Unable to find SyncTextFile for ##${linkId}`);
            const code = link.value?.value;
            if (!code)
                throw new Error(`##${link.id} must have value`);
            return code;
        });
        const functionExpressionString = ts
            .transpileModule(code, {
            compilerOptions: {
                module: ts.ModuleKind.ESNext,
                sourceMap: true,
                target: ts.ScriptTarget.ESNext,
            },
        })
            .outputText.replace("export {}", "");
        const fn = eval(functionExpressionString);
        const result = await fn(...args);
        return result;
    }
};
export {};
//# sourceMappingURL=parse-it-insert-handler.js.map